function [ model, Param ] = Estim_SLI_MLE(dd, locs, zval, kernel, K, method, Param0, LB, UB )
%   Estim_SLI_MLE determines the SLI parameters using maximum likelihood
%
%================ INPUT PARAMETERS ========================================
%
%   dd:         Dimensionality
%   LOCS:       Training set locations
%   ZVAL:       Training set values
%   KERNEL:     Kernel function used
%   K:          Neighbor order 
%   METHOD:     MLE estimation method ('MLE', 'GLO_MLE', 'MLE2',  'GLO_MLE2')
%               MLE:    Standard MLE with Negative log-likelihood (NLL)
%               minimization (four parameters)
%               GLO-MLE: NLL optimization using global search
%               MLE2:   NLL optimization with explicit lambda (scale
%               factor) optimization (three parameters)
%               GLO-MLE2:   NLL optimization with explicit lambda (scale
%               factor) optimization (three parameters) using global search
%   PARAM0:     Initial estimates for the SLI parameters [mz lambda c1 mu ]
%   LB:         Lower bounds on the SLI parameters
%   UB:         Upper bounds on the SLI parameters
%
%================ OUTPUT PARAMETERS =======================================
%   PROGRAM OUTPUTS INCLUDE THE FOLLOWING
%
%   PARAM:      Vector containing the SLI parameters: mz, lambda, c1, mu
%   MODEL:      Contains precision and distance matrices 
%               model.J = J: Precision matrix
%               model.Jsub = Jsub: Precision sub-matrix
%               model.distms = distms: Distance matrix
%               model.h1 = h1: Vector of bandwidths
%               model.Den = Den:    Denominator in the precision matrix
%
%   MLE IS USED FOR PARAMETER INFERENCE
%
%==========================================================================
%
%  Copyright (C) 2020 Dionisis Hristopulos
%   email: dionisi@mred.tuc.gr
%
%  This program is free software; you can redistribute it and/or modify it
%  under the terms of the GNU General Public License as published by the
%  Free Software Foundation (version 2 of the License or  later version).
%
%  This program is distributed WITHOUT ANY WARRANTY.
%  See the GNU General Public License for more details.
%==========================================================================
%   FUNCTIONS CALLED: calc_dist_g, sli_mle_fmincon, sli_mle2, find_lambda 
%               estim_J2_gra 
%==========================================================================
%   SLI PARAMETER INFERENCE
%==========================================================================

[rr, h1]  = calc_dist_g(dd, locs, K); 

distms{1} = rr; 
distms{2} = h1; 

clear rr;

%==========================================================================
%   Estimate SLI model parameters - Use maximum likelihood
%==========================================================================

options = optimoptions('fmincon');

% The following is used for the test runs on simulated data
% options = optimoptions(options,'MaxIter', 10000, 'MaxFunEvals', 10000, 'TolFun', 1E-4,...
%     'TolX', 1E-4, 'algorithm', 'interior-point');


% The following is used for the SLI data
options = optimoptions(options, 'MaxIter', 10^3, 'MaxFunEvals', 10^3, 'TolFun', 1E-3,...
    'TolX', 1E-4, 'algorithm', 'interior-point');

switch method
    
    case 'MLE'
%==========================================================================
%   SLI PARAMETER INFERENCE (MLE)
%==========================================================================
[Param, fval, exitflag, output] = fmincon(@(Param) sli_mle_fmincon(Param, zval,...
   distms, kernel), Param0, [], [], [], [], ...
    LB, UB, [], options);
        
    case 'GLO_MLE' % global optimization
        zval =zval(:);
        fopt = @(x) sli_mle_fmincon(x, zval, distms, kernel);
        
        problem = createOptimProblem('fmincon', 'objective', fopt, 'x0',...
            Param0, 'lb', LB, 'ub', UB, 'options', options);
        
        gs = GlobalSearch;
        [Param, fval, exitflag, output] = run(gs, problem);
        
    case 'MLE2' % explicit lambda optimization
        Param1_0 = [Param0(1) Param0(3) Param0(4)];
        LB2 = LB; LB2(2) = [];
        UB2 = UB; UB2(2) = [];
                
        [Param1, fval, exitflag, output] = fmincon(@(Param1) sli_mle2(Param1, zval,...
            distms, kernel), Param1_0, [], [], [], [], LB2, UB2, [], options);
        
        %   Determine the parameter lambda
        
        lambda = find_lambda(Param1, zval, distms, kernel);
        
        Param(1) = Param1(1);
        Param(2) = lambda;
        Param(3) = Param1(2);
        Param(4) = Param1(3);

    case 'GLO_MLE2' % global optimization & explicit lambda optimization
        zval =zval(:);
        Param1_0 = [Param0(1) Param0(3) Param0(4)];
        LB2 = LB; LB2(2) = [];
        UB2 = UB; UB2(2) = [];
        
        fopt = @(x) sli_mle2(x, zval, distms, kernel);
        
        problem = createOptimProblem('fmincon', 'objective', fopt, 'x0',...
            Param1_0, 'lb', LB2, 'ub', UB2, 'options', options);
        
        gs = GlobalSearch;
        [Param1, fval, exitflag, output] = run(gs, problem);
        %   Determine the parameter lambda
        
        lambda = find_lambda(Param1, zval, distms, kernel);
        
        Param(1) = Param1(1);
        Param(2) = lambda;
        Param(3) = Param1(2);
        Param(4) = Param1(3);
end
%---------------------------------------------

%==========================================================================
%
%   Evaluate the precision matrix

[J, Jsub, Den] = estim_J2_gra(Param(3:end), distms, kernel); 

model.J = J; 
model.Jsub = Jsub;
model.distms = distms;
model.h1 = h1;
model.Den = Den;
end

