% Main_examples.m is a script that can be used to study the different
% datasets provided in this folder using SLI.  
% The user can choose between six different datasets.
% 
% The parameters of the SLI (Spherical Kernel, K = 3 Neighbors) are
% estimated and then a map is created over a square 100 x 100 prediction
% grid.
%   
%   In order to run the deisred example specify the appropriate number
%   (1-6) on Line 82 of the code.
%
%================ INPUT PARAMETERS ========================================
%   data:           Structure containing fields data.locs and data.val
%   data.locs:      Training set locations (array N x dd)
%   data.val:       Field values at the training set points (N x 1)
%   modelK:         Structure containing fields kernel and nei
%   modelK.kernel:  Kernel function (help kernel_s for a list of types)
%   model.nei:      Neighbor order (e.g., use model.nei = 3)
%================ OUTPUT PARAMETERS =======================================
%   modelsli:   Structure containing SLI model 
%   Param_sli:  SLI parameters [mz lambda c1 mu] 
%   tic_map:    Time required to construct the map grid
%   xi:         X coordinates of map grid  (matrix 100 x 100)
%   yi:         Y coordinates of map grid  (matrix 100 x 100)
%   zi:         SLI prediction at map grid nodes
%   zistd:      SLI standard deviation at map grid nodes
%==========================================================================
%
%  Copyright (C) 2020  Dionisis Hristopulos, Andreas Pavlides, Vasiliki
%  Agou
%  email: dchristopoulos@ece.tuc.gr
%
%  This program is free software; you can redistribute it and/or modify it
%  under the terms of the GNU General Public License as published by the
%  Free Software Foundation (version 2 of the License or  later version).
%
%  This program is distributed WITHOUT ANY WARRANTY.
%  See the GNU General Public License for more details.
%
%==========================================================================
%   OTHER FUNCTIONS CALLED: Main_sli_mle, Estim_SLI_mle, sli_mle, 
%             Main_loocv_sli, Main_sli_map, Predict_SLI, crosval_mD
%==========================================================================
%   PARAMETRIZATION
%==========================================================================
clear variables; close all; clc;

% ================ CHOOSE BETWEEN EXAMPLES 1-6 ============================
% Notes on the datasets:
% 
% Example 1. Dataset: ZincData.mat Zinc data at 155 scattered locations  
% from the Meuse dataset.  Mapping on Meuse data grid. 
% 
% Example 2. Dataset: Example_2D_map involves 600 scattered points inside a 
% 10 x 10 square domain.
% The data were simulated using covariance  matrix decomposition. 
% The spherical covariance model was used with the following parameters
% variance = 3, range = 1.7, nugget effect = 0.09.
%
% Example 3. Dataset:  Example_3D contains values at 2000 scattered points  
% inside a rectangular 100 x 100 x 20 domain.
% RFsim  with parameters sill = 1, nugget =0, mean = 0,  corrmodel="exp_exp",  
% scale_s=20,  scale_t=10, sill=5.
% (Separable model, spatial exponential model and temporal exponential model) 
% R package "CompRandFld", command: RFsim
%
% Example 4. Dataset: SimUncExpData contains values at 2500 scattered points 
% inside a 20 x 20 square grid domain. The exponential covariance was 
% used with model parameters: variance=5, scale=10, nugget(var=1), and 
% trend (mean=10.5). R package RandomFields, command: RFsimulate.
%
% Example 5. Dataset: SimConExpData contains values at 4900 scattered points
% inside a 100 x 100 square grid domain. The exponential covariance was used 
% with model parameters: variance=5, scale=15, nugget(var=1), and 
% trend(mean=10).  R package RandomFields, command: RFsimulate.
% 
% Example 6. Dataset:  SimConWhittleData4 contains values at 4900 scattered 
% points inside a 100 x 100 square grid domain. The Whittle covariance was
% used with model parameters: nu=1, variance = 2, scale = 10,  
% Aniso=matrix(nc=2, c(1.5, 3, -3, 4)) and trend(mean=10). 
% R package RandomFields, command: RFsimulate.

Example = 3;

switch Example
    case 1
        load ZincData.mat
        data = ZincData;
%         data.val = log(data.val);
        plocs = ZincData.plocs;
    case 2
        load Example_2D_map.mat
    case 3 
        load Example_3D.mat 
        data = Data42R2;
    case 4
        load SimUncExpData.mat
        data = SimUncExpData;
    case 5
        load SimConExpData.mat
        data = SimConExpData;
    case 6
        load SimConWhittleData4.mat
        data = SimConWhittleData;
end

%======================== Initialize parameters ===========================
modelK.kernel = 'Sphe';
modelK.nei = 3;
Param0 = [mean(data.val  ) 100 1 1.5]; % [mz la c1 mu] Initial guesses

%====================== Estimate model parameters =========================
[Param_sli, modelsli] = Main_sli_mle(data, modelK, Param0);

%==================== Leave One Out Cross Validation ======================
[ zp ] = Main_loocv_sli( data, modelsli, Param_sli );

%=========================== Validate results =============================
fig = 1; % fig = 0; don't plot % fig = 1; plot % fig = 2; plot and save
[err,val] = crosval_mD(data.val,zp.est,10,'test',['test_code_Example' num2str(Example)],fig);

%============== create a 100 x 100 prediction grid ========================
if size(data.locs,2)==2
    x = linspace(min(data.locs(:,1)),max(data.locs(:,1)));
    y = linspace(min(data.locs(:,2)), max(data.locs(:,2)));
    [xi,yi] = meshgrid(x,y);
    
%=============================== Create the map ===========================
    [tic_map, zi, zistd] = Main_sli_map(data, modelsli, Param_sli, xi, yi);
else % 3d data
end
    
    


