function [tic_map, zi, zistd] = Main_sli_map(data, modelsli, Param_sli, xi, yi)
%   This function generates maps of SLI predictions and standard deviations
%   at the nodes of a map grid [xi, yi] based on interpolation of "data".
%   The SLI estimation function Main_sli_mle_needs to run first in order to
%   estimate the SLI model. 
%   WARNING:    This works for dd=2 (two space dimensions)
%================ INPUT PARAMETERS ========================================
%
%   data:       Structure containing fields data.locs and data.zs.
%   data.locs:       Training set locations (array N x dd)
%   data.zs:         Field values at the training set points (N x 1)
%   modelsli:   Structure containing SLI model (from Main_sli_mle)
%   Param_sli:  SLI parameters [mz lambda c1 mu] (derived from Main_sli_mle)
%   xi:         X coordinates of map grid  (matrix N1 x N2)
%   yi:         Y coordinates of map grid  (matrix N1 x N2)
%
%================ OUTPUT PARAMETERS =======================================
%
%   tic_map:    Time required to construct the map grid
%   zi:         SLI prediction at map grid nodes
%   zistd:      SLI standard deviation at map grid nodes
%
%==========================================================================
%
%==========================================================================
%   Functions used include the following: calc_dist_g, estim_J2_gra, 
%   Predict_SLI
%==========================================================================
%
%  Copyright (C) 2020  Dionisis Hristopulos, Andreas Pavlides, Vasiliki
%  Agou
%  email: dchristopoulos@ece.tuc.gr
%
%  This program is free software; you can redistribute it and/or modify it
%  under the terms of the GNU General Public License as published by the
%  Free Software Foundation (version 2 of the License or  later version).
%
%  This program is distributed WITHOUT ANY WARRANTY.
%  See the GNU General Public License for more details.
%==========================================================================

kernel = modelsli.kernel;
K = modelsli.nei;

locs = data.locs;
dd = size(locs, 2); % Dimension of space
zs = data.val; 

%==========================================================================
tic;  % Time for map generation

Pmap = [ xi(:) yi(:)];

Np = length(Pmap);
Ns = length(zs); 
nseg = floor(Np/Ns);
Zpi = zeros(Np,1);
Slistd = zeros(Np, 1);

%   To predict the map grid values segment the grid in several segments
%   with size equal to the sampling size Ns
for i =1: nseg
    ip = 1+ (i-1)*Ns :1 : i*Ns;
    Pmi = Pmap(ip,:); 
    [ Zphat, svar ] = Predict_SLI(dd, locs, zs, Pmi, kernel, K, ...
        modelsli, Param_sli );
    Zpi(ip) = Zphat;
    Slistd(ip) = sqrt(svar);
end

ip = ip(end)+1 :1 : Np;
Pmi = Pmap(ip,:); 
[ Zphat, svar ] = Predict_SLI(dd, locs, zs, Pmi, kernel, K, ...
    modelsli, Param_sli );
Zpi(ip) = Zphat;
Slistd(ip) = sqrt(svar);

zi = reshape(Zpi, size(xi));
zistd = reshape(Slistd, size(xi));

tic_map = toc;
%==========================================================================

figure; 
p1 = surf(xi, yi, zi);
view(2); 
shading flat
axis equal;
colorbar('FontSize',14);
colormap('parula')
axis tight;

figure; 
p2=surf(xi, yi, zistd);
view(2); 
shading flat
axis equal;
colorbar('FontSize',14);
colormap('parula')

