function [Param_sli, modelsli] = Main_sli_mle(data, modelK, Param0, lb, ub)
%  Main_sli_mle estimates the SLI parameters for the specified dataset and 
%   kernel function using maximum likelihood estimation. 
%================ INPUT PARAMETERS ========================================
%
%   data:       Structure containing fields data.locs and data.zs.
%   data.locs:       Training set locations (array N x dd)
%   data.zs:         Field values at the training set points (N x 1)
%   modelK:     Structure containing fields kernel and nei
%   modelK.kernel:  Kernel function (help kernel_s for a list of types)
%   model.nei:      Neighbor order (e.g., use model.nei = 3)
%
%================ OPTIONAL PARAMETERS =====================================
%   Param0: Initial estimates for SLI paramaters [mz lambda c1 mu]
%   lb:  Lower bounds for SLI parameters
%   ub:  Upper bounds for SLI parameters
%
%================ OUTPUT PARAMETERS =======================================
%
%   modelsli:   Structure containing SLI model 
%   Param_sli:  SLI parameters [mz lambda c1 mu] 
%==========================================================================
%
%  Copyright (C) 2020  Dionisis Hristopulos, Andreas Pavlides, Vasiliki
%  Agou
%  email: dchristopoulos@ece.tuc.gr
%
%  This program is free software; you can redistribute it and/or modify it
%  under the terms of the GNU General Public License as published by the
%  Free Software Foundation (version 2 of the License or  later version).
%
%  This program is distributed WITHOUT ANY WARRANTY.
%  See the GNU General Public License for more details.
%
%==========================================================================
%   OTHER FUNCTIONS CALLED: Estim_SLI_mle, sli_mle, Predict_SLI 
%==========================================================================
%   PARAMETRIZATION
%==========================================================================

method = 'MLE'; % Only option MLE enabled

kernel = modelK.kernel;
K = modelK.nei;

locs = data.locs;
dd = size(locs, 2);
z = data.val; 

%--------------------------------------------------------------------------
%   Parameters used for SLI estimation
%--------------------------------------------------------------------------

if exist('Param0', 'var')~=1
    Param0 = [mean(z) 100 1 1.5]; % [mz la c1 mu] Initial guesses
end
if exist('lb', 'var')~=1
    lb = [ mean(z)-2*std(z)  eps       eps   0.5];
end
if exist('ub', 'var')~=1
    ub = [ mean(z)+2*std(z)  inf       inf    15];
end

tic;

[ modelsli, Param_sli ] = Estim_SLI_MLE(dd, locs, z, kernel, K, method, Param0, lb, ub );
[nll, ~, ~] = sli_mle(Param_sli, z, modelsli.distms, kernel);
modelsli.nll = nll;
modelsli.kernel = kernel;
modelsli.nei = K;

%   PARAMSLI:      Vector containing the SLI parameters: mz, lambda, c1, mu
%   MODELSLI:   Contains precision and distance matrices 
%               modelsli.J = J: Precision matrix
%               modelsli.Jsub = Jsub: Precision sub-matrix
%               modelsli.distms = distms: Distance matrix
%               modelsli.h1 = h1: Vector of bandwidths
%               modelsli.Den = Den:    Denominator in the precision matrix   







