function [ rr, hx1 ] = calc_dist_cross_g(dd, locs, plocs, K)
%   CALC_DIST_CROSS_G Calculates the matrix of distances and local bandwidths
%   aspect ratios for prediction and sampling point sets. 
%   The common scaling factor MU is required to fully determine the SLI
%   bandwidths. MU is determined in the process of SLI parameter inference
%
%==========================================================================
%   Input parameters
%==========================================================================
%   dd:     Dimensionality
%   locs:   Locations of sampling points (N x DD) 
%   plocs:   Locations of prediction points (P x DD)
%   K:      how many nearest neighbors to take into consideration (integer)
%
%==========================================================================
%   Output parameters
%==========================================================================
%
%   rr:      Matrix of sampling - prediction points pair distances ( P x N) 
%   hx1:     Local gradient bandwidth ratios at prediction points (P x 1)            
%==========================================================================
%
%   FUNCTIONS USED
%
%   PDIST2:     Efficient calculation of pair distances between the sampling
%               and prediction set points
%   KNNSEARCH:  Estimation of near neighbors
%========================= EXAMPLE ========================================
%
%   locs = rand(10,2);  plocs = rand(3,2); 
%   [ r, h1 ] = calc_dist_cross_g(2, locs, plocs, 3);
%
%==========================================================================
%
%  Copyright (C) 2014  Dionisis Hristopulos
%   email: dionisi@mred.tuc.gr
%
%  This program is free software; you can redistribute it and/or modify it
%  under the terms of the GNU General Public License as published by the
%  Free Software Foundation (version 2 of the License or  later version).
%
%  This program is distributed WITHOUT ANY WARRANTY.
%  See the GNU General Public License for more details.
%
%==========================================================================
if size(plocs, 2) ~= dd
    plocs = plocs' ; 
end

if size(locs, 2) ~= dd
    locs = locs' ; 
end

Nlocs = length(locs); 
if Nlocs <= K
    disp('Number of sampling locations should be > K');
    rr = nan; hx1 = nan;
    return;
end

rr = pdist2(plocs, locs); 

%==========================================================================
%   Determine the local bandwidth aspect ratios
%==========================================================================
%   Use nearest neighbors for infinitely supported kernels and second
%   nearest neighbors for compactly supported kernels
%==========================================================================


switch dd
    case 1
        x=locs(:); y = plocs(:);
        [~, Dx] = knnsearch(x, y, 'k', K);

    case 2
        if size(locs, 2) ~= 2
            disp('Dimension mismatch');
            return;
        end;
        
        [~, Dx] = knnsearch(locs, plocs, 'k', K);
        
    otherwise
        [~, Dx] = knnsearch(locs, plocs, 'k', K);
        
end;

hx1 = Dx(:, K);

end

