function [ r, h1 ] = calc_dist_g(dd, locs, K)
%   CALC_DIST_G Calculates the distance matrix and local bandwidths aspect
%   ratios. The common scaling factor MU is required to fully determine
%   bandwidths
%
%================ INPUT PARAMETERS ========================================
%   DD:         Dimension index (integer)
%   LOCS:       Array containing the coordinates of points ( N x DD)
%   K:          Number of near neighbors to take into consideration (integer)
%               NOTE: for kernels 'Expo' and 'Gaus' works best with K = 2
%   
%================ OUTPUT PARAMETERS =======================================
%   RR:         Matrix of point-point distances ( N x N)
%   H1:         Vector of gradient bandwidth ratios ( N x 1)
%
%========================= EXAMPLE ========================================
%
%   locs = rand(10,3);  [ r, h1 ] = calc_dist_g(3, locs, 'Quad', 3);
%
%========================= EXAMPLE ========================================

%  Copyright (C) 2017  Dionisis Hristopulos
%   email: dionisi@mred.tuc.gr
%
%  This program is free software; you can redistribute it and/or modify it
%  under the terms of the GNU General Public License as published by the
%  Free Software Foundation (version 2 of the License or  later version).
%
%  This program is distributed WITHOUT ANY WARRANTY.
%  See the GNU General Public License for more details.
%
%   This software bundle also includes code developed by other people. 
%
%   DISTMAT: Developed by Joseph Kirk
%
%==========================================================================
N = length(locs);

if N <= K
    disp('Number of sampling locations should be > K');
    r = nan; h1 = nan;
    return;
end


if size(locs,2) ~= dd
    locs = locs'; 
end

r = zeros(N); 
h1 = zeros(N,1); 

if N <=30, opt=1;
elseif (30< N) && (N<=300), opt=2;
else opt=3;
end

switch dd
    case rem(dd,1)~=0
        disp('Non-integer dimension');
        return;
    case dd<0
        disp('Negative dimension');
        return;
    case 2
        r = distmat(locs, opt);
    case 1
        xs=locs;
        r = abs( xs * ones(1, N) - ones(N, 1) * xs' );
    otherwise
        r = distmat(locs, opt);
end;

%==========================================================================
%   Determine the local bandwidth aspect ratios
%==========================================================================
%   Use nearest neighbors for infinitely supported kernels and second
%   nearest neighbors for compactly supported kernels
%==========================================================================

% if strcmp(kernel,'Expo') || strcmp(kernel,'Gaus')
%     K=2;
% else
%     K=4;
% end
K = K+1; % +1 because K = 1 is the same point --> distance 0

switch dd
    case 1
        x = locs(:);
        [~, D] = knnsearch(x, x, 'k', K);
    case 2
        if size(locs, 2)~=2
            disp('Dimension mismatch');
            return;
        end;
        
        [~, D] = knnsearch(locs, locs, 'k', K);
        
    otherwise
        [~, D] = knnsearch(locs, locs, 'k', K);
        
end;


h1 = D(:,K);


end

