function [err,val] = crosval_mD(zs, zp, ifig, tit_fig, fig_name, fig)
%  CROSVAL_MD calculates cross validation measures between the sampled
%   values ZS and the predictions ZP
%
%================ INPUT PARAMETERS ========================================
%
%   zs:         Z values of sample
%   zp:         Predictions
%   ifig:       Initial value of figure number
%   tit_fig:    Figures' title
%   fig_name:   Prefix of the name of figure files
%   fig:        0 don't show figures, else show
%
%==========================================================================
%   Output parameters
%==========================================================================
%
%   val:    array of validation measures
%   err:    array of prediction errors
%
%   More specifically, VAL contains the following measures
%
%   ME:     Mean error (bias)
%   MAE:    Mean absolute error
%   MARE:   Mean absolute relative error (INF if z contains zeros)
%   RMSE:   Root mean square error
%   RMSRE:  Root mean square relative error (INF if z contains zeros)
%   R2:     Linear correlation coefficient
%   RS:     Spearman (rank) corrrelation coefficient
%   ErrMin: Minimum error
%   ErrMax: Maximum error
%
%==========================================================================
%
%   FUNCTIONS USED
%
%========================= EXAMPLE ========================================
% 
% [err,val] = crosval_mD(rand(100,1), rand(100,1), 19, 'Validation set CV', 'testing',1);
% 
%==========================================================================
%  Copyright (C) 2020  Dionisis Hristopulos
%   email: dionisi@mred.tuc.gr
%
%  This program is free software; you can redistribute it and/or modify it
%  under the terms of the GNU General Public License as published by the
%  Free Software Foundation (version 2 of the License or  later version).
%
%  This program is distributed WITHOUT ANY WARRANTY.
%  See the GNU General Public License for more details.
%
%==========================================================================

nfig=1;

Zhat=reshape(zp,size(zs));

ME = nanmean(Zhat -zs);
MAE = nanmean(abs(Zhat -zs));
MARE = nanmean(abs((Zhat -zs)./zs));
RMSE = sqrt(nanmean((Zhat -zs).^2));
RMSRE = sqrt(nanmean(((Zhat -zs)./zs).^2));
RP = corr(Zhat,zs);
RS = corr(Zhat,zs,'type','Spearman');

ErrMin = min(Zhat - zs); 
ErrMax = max(Zhat - zs); 

val = [ME MAE MARE RMSE RMSRE RP RS ErrMin ErrMax];
val = array2table(val);
val.Properties.VariableNames = {'ME'; 'MAE'; 'MARE'; 'RMSE'; 'RMSRE'; 'RP'; 'RS'; 'ErrMin'; 'ErrMax'};
err = Zhat-zs;

if fig~=0
%============= Histogram of Prediction Errors =============================
    figure((ifig-1)*nfig+1);
    hist(err);
    xlabel('Prediction error');
    ylabel('Frequency');
    title(sprintf(tit_fig), 'interpreter', 'none');
    histpub;
    
    err(err==0) = eps;
    figname=[ fig_name '_'  tit_fig '_Error'];
    
    if fig==2
        saveas(gcf,figname,'png');
    end
%======== Scatter plot of predictions vs real values ======================
    figure((ifig-1)*nfig+2);
    
    minz = min(min(Zhat),min(zs));
    maxz = max(max(Zhat),max(zs));
    
    scatter(Zhat,zs,'.');
    axis([minz maxz minz maxz]);
    scatter(Zhat,zs,abs(err),abs(err));
    axis([minz maxz minz maxz]);
    hold on;
    li=line([minz maxz], [minz maxz]);
    set(li,'LineWidth',2);
    set(li,'Color','r');
    
    xlabel('Predictions');
    ylabel('Validation');
    axis('image');
    pbaspect([1 1 1])
    box on
    figname=[ fig_name '_'  tit_fig '_Scatter_Pred'];
    
    if fig==2
        saveas(gcf,figname,'png');
    end
%========= Histograms of True and Predicted values ========================
    figure((ifig-1)*nfig+3);
    hist([zp(:) zs(:)]);
    xlabel('Z values');
    ylabel('Frequency');
    legend('Predictions','Sample values','Location','Best');
    alpha(0.5);
    title(sprintf(tit_fig), 'interpreter', 'none');
    
    axis tight;
    figname=[ fig_name '_'  tit_fig '_Histograms'];

    if fig==2
        saveas(gcf,figname,'png');
    end
end

