function [Jmat, Jsub, Den] = estim_J2_gra(Param1, distms, kernel)
%   ESTIM_J2_GRA calculates the SLI precision matrix between sampling points
%==========================================================================
%   Input parameters
%==========================================================================
%   Param1:  SLI parameters (c1, mu) 
%   distms: Cell containing the matrix of pair distances and the vector of
%           gradient bandwidths
%   kernel: Type of kernel function that will be used
%
%==========================================================================
%   Output parameters
%==========================================================================
%
%	Jmat:   SLI Precision matrix
%   Jsub:   SLI precision submatrices
%   Den:    Denominator of gradient kernel averages
%
%==========================================================================
%	EXAMPLE
%==========================================================================
%
%
%==========================================================================
%
%   FUNCTIONS USED
%
%   KERNEL_S:   Kernel function
%
%==========================================================================
%
%  Copyright (C) 2020 Dionisis Hristopulos
%   email: dionisi@mred.tuc.gr
%
%  This program is free software; you can redistribute it and/or modify it
%  under the terms of the GNU General Public License as published by the
%  Free Software Foundation (version 2 of the License or  later version).
%
%  This program is distributed WITHOUT ANY WARRANTY.
%  See the GNU General Public License for more details.
%==========================================================================
%   Notes: Unlike previous versions of the code (Estim_J2_g) this version
%   assumes that the dimension factor d is absorbed in c1. This means that
%   values of c1 obtained with the old code should be multiplied with 2.

c1 = Param1(1);
mu = Param1(2); 

r = distms{1}; 

h1 = mu * distms{2}; 

N = length(h1); 

Jsub = cell(2,1);
Jsub{1} = zeros(N,N);
Jsub{2} = zeros(N,N); 

%==========================================================================
%   Build the Precision matrix
%==========================================================================
J0=sparse(eye(N)/N); %sparse

%===================Gradient submatrix=====================================

hs1=repmat(h1(:),1,N);
Num1 = sparse(kernel_s( r./hs1, kernel));
Den1 = sum(sum((Num1)));

v1=sum(Num1,2); 
v2=sum(Num1,1); 

u1= -(Num1 + Num1') + diag( v1 + v2');

J1 =  u1 ./ Den1;

if any(isnan(J1)), disp('Nan Values in J1'); 
end

%==========================================================================

Jmat = J0 +  c1 * J1;

Jsub{1} = J0; 
Jsub{2} = c1 * J1 ; 

if any(isinf(Jmat)), 
    disp('Inf Values in J'); 
end

Den = Den1; 

end

