function [JPS, JPP] = estim_JP_gra(Param, distmsp, h1, Den, kernel)
%   ESTIM_JP_GRA calculates the SLI precision matrix between sampling and 
%   prediction points
%==========================================================================
%   Input parameters
%==========================================================================
%   Param:  SLI parameters (c1, mu) 
%   distmsp: Cell containing the matrix of pair distances and the matrix of
%            gradient bandwidths calculated for pairings between sampling 
%            and prediction points
%   Den:    Denominator components between sampling points for the gradient
%   kernel: Type of kernel function that will be used
%
%==========================================================================
%   Output parameters
%==========================================================================
%
%	JPS:   SLI Precision matrix between sampling points and prediction
%           points
%	JPP:   SLI Precision matrix elements for prediction - prediction
%           points
%
%==========================================================================
%	EXAMPLE
%==========================================================================
%
%
%==========================================================================
%
%   FUNCTIONS USED
%
%   KERNEL_S:   Kernel function
%
%==========================================================================
%
%  Copyright (C) 2013 Dionisis Hristopulos
%   email: dionisi@mred.tuc.gr
%
%  This program is free software; you can redistribute it and/or modify it
%  under the terms of the GNU General Public License as published by the
%  Free Software Foundation (version 2 of the License or  later version).
%
%  This program is distributed WITHOUT ANY WARRANTY.
%  See the GNU General Public License for more details.
%==========================================================================
%   The parameter DISPLAY is used to control the display of intermediate
%   calculation results. DISPLAY=FALSE suppresses the output of results,
%   whereas DISPLAY=TRUE enables the display 


DenN1 = Den; % This denominator term is due only to the sampling points

c1 = Param(1);
mu = Param(2); 

%=====================Increment & Bandwidth selection======================

%==========================================================================
%   First, we calculate the coefficients that are rooted at the PREDICTION
%   points, i.e., whose bandwidth is determined from the prediction points
%   neighborhoods. 

rsp = distmsp{1}; 

hp1 = mu * distmsp{2}; 

hp1(hp1==0) = eps;

%==========================================================================

P = length(hp1) ; 
N = length(h1) ; 

%==========================================================================
hpp1 = repmat(hp1(:),1,N);

%==========================================================================
%   Then, we calculate the coefficients that are rooted at the SAMPLING
%   points, i.e., whose bandwidth is determined from the SAMPLING points
%   neighborhoods. 

hs1 = mu * h1;
hs1 = hs1(:); 

hss1 = repmat(hs1', P, 1);  
 
%==========================================================================
%   Build the Precision matrix -  
%==========================================================================

J0 = sparse(0); 
% Since the prediction and sampling sets are disjoint the diagonal term is
% zero

%===================Gradient submatrix=====================================

Num1p = sparse(kernel_s(rsp./hpp1,kernel));
Num1s = sparse(kernel_s(rsp./hss1,kernel));

Den1 = repmat(DenN1 + sum(Num1p,2) + sum(Num1s,2), 1,N);

u1= - (Num1p + Num1s) ;

J1 = sparse(u1 ./ Den1);

if any(isnan(J1)), disp('Nan Values in J1'); 
end


%==========================================================================

JPS = J0 +  c1 * J1; 

if any(isinf(JPS)), disp('Inf Values in J'); 
end

%==========================================================================

JPP = 1/N -  sum(JPS,2); 

end

