function y=kernel_s(r,kernel,nu)
%==========================================================================
%   The function KERNEL_S computes the kernel function at given distance
%   'r'
%====================   INPUT ============================================
%   r:          Vector (or matrix) of distances
%   KERNEL:     Type of kernel funcion used in the estimate of the 
%                   sample constraints
%               Supported kernel functions 
%               Kernel choices: 
%               Epanechnikov:    'Tria' (bitriangular)
%               Quadratic:      'Quad'
%               Tricubic:       'Tric'
%               Exponential:    'Expo'
%               Gaussian:       'Gaus'
%               Cauchy:         'Cauc'
%               Spherical:      'Sphe'
%               Cosine:         'Cosi'
%               Sine wave:      'Sinc'
%               Triangular:     'Trie'
%               Dirac:          'Dira'
%               Uniform:        'Unif'
%               Biweight:       'Biwe'
%               Matern:         'Mate'
%
%   NU:     Smoothness parameters (used olny for Matern kernel)
%
%====================   OUTPUT ============================================
%
%   y:  Values of the kernel function (vector or matrix, depending on r)
%
%==========================================================================
%
%  Copyright (C) 2013 Samuel Elogne and Dionisis Hristopulos
%   email: dionisi@mred.tuc.gr
%
%  This program is free software; you can redistribute it and/or modify it
%  under the terms of the GNU General Public License as published by the
%  Free Software Foundation (version 2 of the License or  later version).
%
%  This program is distributed WITHOUT ANY WARRANTY.
%  See the GNU General Public License for more details.
%==========================================================================
%	REFERENCES (for some of the kernel expressions and applications with
%	Spartan spatial random fields
%==========================================================================
%
%   D. T. Hristopulos and S. Elogne, Computationally efficient spatial 
%   interpolators based on Spartan spatial random fields, 
%   IEEE Transactions on Signal Processing, 57(9), 3475-3487 (2009). 
%
%   M. Zukovic and D. T. Hristopulos, The method of normalized correlations: 
%   a fast parameter estimation method for random processes and isotropic 
%   random fields that focuses on short-range depende nce, 
%   Technometrics, 15(2), 173-185 (2009).
%==========================================================================

if strcmp(kernel,'Sinc')

    y=(1*(r==0)+sin(r)./(r+(r==0)));
    
elseif strcmp(kernel,'Cosi')
    y=cos(r);
    
elseif strcmp(kernel,'Tria')
    y=((1-abs(r)).^2).*(abs(r)<=1) ;

elseif strcmp(kernel,'Tric')
%   Tricubic Kernel
    y=((1-abs(r).^3).^3).*(abs(r)<=1);

elseif strcmp(kernel,'Quad')
    y=((1-r.^2)).*(abs(r)<=1);

elseif strcmp(kernel,'Gaus')
    y=exp(-r.^2);

elseif strcmp(kernel,'Expo')
    y=exp(-abs(r));
    
elseif strcmp(kernel,'Exps')
    y=exp(-(abs(r)).^(0.57));
    
elseif strcmp(kernel,'Trie')
y=((1-abs(r))).*(abs(r)<=1) ;

elseif strcmp(kernel,'Triw')
y=((1-abs(r).^2).^3).*(abs(r)<=1) ;

elseif strcmp(kernel,'Biwe')
    y=((1-abs(r).^2).^2).*(abs(r)<=1) ;

elseif strcmp(kernel,'Sphe')
    y=(1-1.5*abs(r)+0.5*abs(r).^3).*(abs(r)<=1);
    
elseif strcmp(kernel,'Cauc')
   % y=(1./(1+r.^2)).*(abs(r)<=10);
    y=(1./(1+r.^2));
    
elseif strcmp(kernel,'Dira')
    y=1.*(abs(r)==0)+0.*(abs(r)~=0);
    
elseif strcmp(kernel,'Unif')
    y=(abs(r) <= 1);
    
elseif strcmp(kernel,'Mate')
    c0=(2^(nu-1))*gamma(nu);
    j=r==0;
    r(j)=eps;
    bes=besselk(nu,r);
    y=(1./c0).*(r.^nu).*bes;
    
else
	sprintf('%s','Unrecognizable Kernel')
end