function [y, y1, y2] = sli_mle(Param, z, distms, kernel)
%   SLI_MLE is the MLE cost functional (Negative Log-Likelihood) used to 
%   determine the SLI parameters 
%==========================================================================
%   Input parameters
%==========================================================================
%   Param:  SLI parameters (mean, lambda, c1, mu)
%	z:      Data values (vector). The data are stored per time slice (i.e.,
%           the spatial index changes first)
%   distms: Cell containing the matrix of pair distances and the matrix of
%           gradient bandwidths [temp_distances spat_dist temp_bandwidths
%           spat_bandwidths]
%   kernel: Type of kernel function that will be used 
%
%==========================================================================
%   Output 
%==========================================================================
%
%	y:      Value of the MLE cost functional (Negative log-likelihood)
%   y1:     Energy of the specific model
%   y2:     Remaining NLL terms
%
%==========================================================================
%	EXAMPLE
%==========================================================================
%
%
%==========================================================================
%   FUNCTIONS USED
%
%   estim_J2_gra:   Precision matrix calculation
%   LOGDET2:        It calculates the logarithm of the determinant of the
%                   large sparse precision mattrix
%==========================================================================
%
%  Copyright (C) 2018 Dionisis Hristopulos
%   email: dionisi@mred.tuc.gr
%
%  This program is free software; you can redistribute it and/or modify it
%  under the terms of the GNU General Public License as published by the
%  Free Software Foundation (version 2 of the License or  later version).
%
%  This program is distributed WITHOUT ANY WARRANTY.
%  See the GNU General Public License for more details.
%==========================================================================
%	REFERENCES (applications to parameter inference for random fields)
%==========================================================================
%
%   The parameter DISPLAY is used to control the display of intermediate
%   calculation results. DISPLAY=FALSE suppresses the output of results,
%   whereas DISPLAY=TRUE enables the display 

if nargin ~= 4
    disp('Incorrect number of arguments in sli_mle'); 
end

N=length(z);

fnan=0; finf=0; 

mz = Param(1);
lambda = Param(2); 
Param1 = Param(3: 4); % Param1 includes SLI parameters without lambda

J = estim_J2_gra(Param1, distms, kernel); 

if any(isnan(J)), disp('Nan Values in J'); 
    fnan=1; 
end

if any(isinf(J)), disp('Inf Values in J'); 
    finf=1; 
end


%==========================================================================
if fnan==1, return; end

zn = z - mz; 
zn = zn(:); 

HH = (zn' * J *zn) /2 ; 

if HH<0 || finf==1, 
    y = realmax; 
else
    
%==========================================================================
%   The following lines show how the Negative Log Likelihood is calculated

% L = exp(-H)/Z => logL = - H - logZ => NLL = -logL = H +logZ
% J= J1/lambda : Z=(2pi)^(N/2) |detJ|^{-1/2} = 
%               (2pi lambda)^(N/2) * (detJ1)^{-1/2}
% => NLL = H + N/2 * log(lambda) - 1/2 log(det J1) = 
%          (zn' * J *zn) /(2 * lambda)  + N/2 * log(lambda) - 1/2 log(det J1)

y1 = HH/lambda; 
y2 = (N*log(2*pi*lambda) - logdet2(J))/2 ;

y = y1 + y2; 

end;


